
# q-expansion of newform 1088.2.a.c, downloaded from the LMFDB on 11 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1088
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-2], [0], [0], [-2], [6], [-1], [-4], [4], [0], [-8], [4], [6], [-8], [-8], [-10], [0], [-12], [-8], [12], [2], [-4], [-16], [10], [-18], [-10], [0], [18], [0], [-6], [8], [2], [-10], [-6], [-6], [8], [2], [2], [-8], [24], [12], [20], [16], [-10], [-20], [-4], [-2], [24], [10], [-6], [-14], [0], [22], [24], [-26], [-16], [8], [-16], [-8], [6], [22], [26], [-12], [24], [-2], [-20], [-32], [-22], [-30], [-10], [6], [24], [-28], [14], [-34], [0], [-6], [12], [-34], [-10], [-14], [-10], [28], [34], [-28], [-8], [14], [-6], [22], [-32], [12], [4], [12], [-12], [38], [12], [34], [14], [16], [-4], [-22], [-10], [8], [2], [10], [14], [-36], [34], [-8], [-14], [-48], [-14], [34], [10], [-24], [14], [-2], [-24], [-8], [-28], [14], [38], [-36], [46], [14], [-42], [32], [-36], [32], [-10], [4], [40], [4], [34], [22], [-2], [18], [-22], [-10], [-2], [38], [-20], [4], [-2], [-6], [48], [8], [-50], [-48], [-48], [-16], [18], [-32], [36], [30], [32], [0], [14], [38], [20], [10], [34], [-56], [8], [10], [-36], [28], [-4], [46], [44], [-4], [-10], [48], [-6], [64], [10], [-10], [-4], [32], [50], [8], [26], [52], [42], [-24], [-10], [-12], [-20], [-58], [-56], [-22], [-30], [20], [-38], [30], [-46], [-38], [4], [2], [0], [12], [-20], [34], [-26], [-44], [42], [24], [44], [-54], [-12], [-42], [-14], [-40], [12], [32], [-18], [40], [2], [-24], [-18], [-70], [24], [-18], [40], [-48], [46], [-6], [-12], [8], [-30], [-58], [6], [-40], [-6], [-56], [-64], [14], [-60], [-6], [24], [-54], [-12], [64], [10], [-6], [-64], [-56], [10], [32], [-56], [-22], [-38], [-48], [-22], [2], [-24], [60], [22], [-28], [22], [-8], [-12], [-20], [-16], [46], [64], [46], [46], [34], [-30], [-16], [-60], [26], [56], [-18], [16], [-48], [38], [-22], [0], [8], [40], [32], [22], [-46], [72], [-2], [-12], [-32], [-30], [48], [-42], [78], [30], [-28], [-56], [8], [6], [34], [-8], [-14], [-54], [8], [14], [-2], [-2], [-60], [-76], [32], [-6], [-28], [-24], [78], [-52], [0], [-58], [-44], [56], [26], [-90], [50], [-26], [82], [-64], [54], [-22], [18], [78], [-36], [52], [-8], [12], [4], [-4], [2], [6], [38], [-30], [10], [88], [38], [-46], [-60], [32], [6], [18], [-52], [8], [-72], [-28], [-8], [30], [-30], [8], [-54], [-10], [72], [14], [38], [-8], [20], [-14], [-72], [-30], [-28], [-74], [-48], [40], [38], [-8], [-50], [-44], [62], [-40], [-64], [38], [48], [54], [46], [42], [80], [54], [36], [18], [34], [-48], [-56], [42], [-2], [24], [-14], [84], [-100], [-4], [40], [-74], [32], [-10], [22], [30], [-4], [-18], [72], [22], [66], [-80], [50], [-2], [-64], [70], [-6], [-38], [-100], [66], [-66], [12], [-84], [80], [38], [-56], [30], [-32], [-12], [60], [-98], [-54], [18], [82], [-8], [-80]]
